//----------------------------------------------------------------------------------------
// les.cpp: Contains functions for linear system solver
//----------------------------------------------------------------------------------------

#include <cmath>
#include "les_solver.H"

static real too_small=1.e-10;

// Diagonal peconditioner for linear system solver
// Parameters: n - dimension of the system; a - system matrix; f - right hand side
inline
void diag_preconditioner(int n, long_real a[], real f[])
{
  for (int i=0, ind=0; i<n; i++)
  {
    long_real s=a[ind];
    if (std::abs(s)>too_small)
    {
      f[i]/=s;
      for (int j=0; j<n; a[ind]/=s, j++, ind++);
    }
  }
}

// LU decomposition for linear system solver
// Parameters: n - dimension of the system; a - system matrix;
int LU_decomposition(int n, long_real a[])
{
  /* Pre-conditioning */
  //diag_preconditioner(n, a, f);

  for (int j=0; j<n; j++)
  {
    for (int i=0; i<=j; i++)
      for (int l=0; l<i; l++)
        a[i*n+j]-=a[i*n+l]*a[l*n+j];
    long_real t=a[j*n+j];
    if (std::abs(t)<too_small)
      return BAD_INDEX;   /* Zero on the diagonal */
    for (int i=j+1; i<n; i++)
    {
      for (int l=0; l<j; l++)
        a[i*n+j]-=a[i*n+l]*a[l*n+j];
      a[i*n+j]/=t;
    }
  }
  return 0;
}

// LU substitution for linear system solver
// Parameters: n - dimension of the system; a - system matrix; u - right hand side/solution
void LU_substitution(int n, const long_real a[], real u[])
{
  /* Forward substitution */
  for (int i=0; i<n; i++)
    for (int j=0; j<i; j++)
      u[i]-=a[i*n+j]*u[j];
  /* Backward substitution */
  for (int i=n-1; i>=0; u[i]/=a[i*n+i], i--)
    for (int j=i+1; j<n; j++)
      u[i]-=a[i*n+j]*u[j];
}
